# -*- coding: utf-8 -*-

import random
import sys
import getopt

LEFT = "LEFT"
RIGHT = "RIGHT"
TOP = "TOP"
DOWN = "DOWN"

def fill_route(route, text):
	width = max([p.x + 1 for p in route])
	height = max([p.y + 1 for p in route])
	board = [[' ' for x in range(width)] for y in range(height)]
	text = list(text)
	text.reverse()
	for point in route:
		board[point.y][point.x] = text.pop()
	return u'\n'.join([u''.join(row) for row in board])
	

def hardness(route):
	turns = 0
	previous_point = route[0]
	previous_direction = None
	for point in route[1:]:
		assert point.is_neighbour(previous_point)
		direction = None
		if point.x > previous_point.x:
			direction = RIGHT
		if point.x < previous_point.x:
			direction = LEFT
		if point.y > previous_point.y:
			direction = DOWN
		if point.y < previous_point.y:
			direction = TOP
		assert not direction is None
		if not previous_direction is None and direction != previous_direction:
			turns += 1
		previous_direction = direction
		previous_point = point
	return float(turns) / float(len(route))

def show_route(route):
	width = max([p.x + 1 for p in route])
	height = max([p.y + 1 for p in route])
	board = [[' ' for x in range(width)] for y in range(height)]
	previous_point = route[0]
	for point in route[1:]:
		assert point.is_neighbour(previous_point)
		znaczek = '?'
		if point.x > previous_point.x:
			znaczek = u'\u2192'
		if point.x < previous_point.x:
			znaczek = u'\u2190'
		if point.y > previous_point.y:
			znaczek = u'\u2193'
		if point.y < previous_point.y:
			znaczek = u'\u2191'
		board[previous_point.y][previous_point.x] = znaczek
		previous_point = point
	return u'\n'.join([u''.join(row) for row in board])

class V:
	"Vector"
	str_pattern = "V(%d, %d)"
	def __init__(self, x, y):
		self.x = x
		self.y = y
	def __str__(self):
		return self.str_pattern % (self.x, self.y)
	def __repr__(self):
		return str(self)
	def __eq__(self, other):
		return self.x == other.x and self.y == other.y
	def __ne__(self, other):
		return not (self == other)
	def __hash__(self):
		return self.x ^ self.y
	def inverted(self):
		return V(-self.x, -self.y)

class P(V):
	"Point"
	str_pattern = "P(%d, %d)"
	def offset(self, vector):
		return P(self.x + vector.x, self.y + vector.y)
	def color(self):
		return self.x % 2 ^ self.y % 2
	def is_neighbour(self, other):
		return abs(self.x - other.x) + abs(self.y - other.y) == 1

def pairs(l1, l2):
	result = []
	for element1 in l1:
		for element2 in l2:
			result.append((element1, element2))
	return result

HORIZONTAL = "HORIZONTAL"
VERTICAL = "VERTICAL"

class Board:
	_cut_direction = None
	_subboards = None
	def __init__(self, width, height, start = None, end = None, offset = P(0, 0)):
		self.width = width
		self.height = height
		self.start = start
		self.end = end
		self.offset = offset
	def possible_cut_directions(self):
		assert self.start != self.end
		result = []
		if self.start.x != self.end.x:
			result.append(VERTICAL)
		if self.start.y != self.end.y:
			result.append(HORIZONTAL)
		if self.width == 2 and self.start.y != self.end.y:
			try:
				result.remove(VERTICAL)
			except ValueError:
				pass
		if self.height == 2 and self.start.x != self.end.x:
			try:
				result.remove(HORIZONTAL)
			except ValueError:
				pass
		return result
	def __str__(self):
		return "Board(%d, %d, %s, %s, %s)" % (self.width, self.height, str(self.start), str(self.end), str(self.offset))
	def __repr__(self):
		return str(self)
	@property
	def cut_direction(self):
		if self._cut_direction is None:
			self._cut_direction = random.choice(self.possible_cut_directions())
		return self._cut_direction
	def abs2rel(self, point_absolute):
		try:
			point_absolute.__iter__
		except AttributeError:
			return point_absolute.offset(self.offset.inverted())
		return [self.abs2rel(p) for p in point_absolute]
	def rel2abs(self, point_relative):
		try:
			point_relative.__iter__
		except AttributeError:
			return point_relative.offset(self.offset)
		return [self.rel2abs(p) for p in point_relative]
	def contains(self, point_absolute):
		"Whether this board contains this point. Point must use absolute coordinates"
		point = self.abs2rel(point_absolute)
		return point.x >=0 and point.y >= 0 and point.x < self.width and point.y < self.height
	def corners(self):
		return set([P(0, 0), P(self.width - 1, 0), P(0, self.height - 1), P(self.width - 1, self.height - 1)])
	def possible_other_points(self):
		assert (self.start is None) ^ (self.end is None)
		this_point = self.start if not self.start is None else self.end
		return [p for p in self.corners() if (p.color() == this_point.color()) == ((self.width * self.height) % 2 == 1) and p != this_point]
	def set_other_point(self, other_point):
		assert (self.start is None) ^ (self.end is None)
		if self.start is None:
			self.start = other_point
		else:
			self.end = other_point
	@property
	def subboards(self):
		if self._subboards is None:
			self._subboards = [None, None]
			size = self.height if self.cut_direction == HORIZONTAL else self.width
			parts = [None, None]
			parts[0] = random.randint(1, size - 1)
			parts[1] = size - parts[0]
			offset_values = [0, 0]
			offset_values[1 if self.cut_direction == HORIZONTAL else 0] = parts[0]
			offsets = (V(0, 0), V(*offset_values))
			points = ({'start': None, 'end': None}, {'start': None, 'end': None})
			for i in (0, 1):
				width = self.width if self.cut_direction == HORIZONTAL else parts[i]
				height = self.height if self.cut_direction == VERTICAL else parts[i]
				start = None
				end = None
				offset = offsets[i]
				self._subboards[i] = Board(width, height, start, end, offset)
			contains_start = self._subboards[0 if self._subboards[0].contains(self.start) else 1]
			contains_end = self._subboards[0 if self._subboards[0].contains(self.end) else 1]
			contains_start.start = contains_start.abs2rel(self.start)
			contains_end.end = contains_end.abs2rel(self.end)
			pairs_of_possible_other_points = pairs(self.subboards[0].rel2abs(self.subboards[0].possible_other_points()), self.subboards[1].rel2abs(self.subboards[1].possible_other_points()))
			choosed_other_points = random.choice([pair for pair in pairs_of_possible_other_points if pair[0].is_neighbour(pair[1])])
			self.subboards[0].set_other_point(self.subboards[0].abs2rel(choosed_other_points[0]))
			self.subboards[1].set_other_point(self.subboards[1].abs2rel(choosed_other_points[1]))
		return self._subboards
	def route(self):
		if self.width == 1 or self.height == 1:
			assert self.width !=1 or self.height != 1
			if self.width == 1:
				assert (self.start.y == 0 or self.start.y == self.height - 1) and (self.end.y == 0 or self.end.y == self.height - 1)
				if self.start.y < self.end.y:
					result = [P(0, y) for y in range(0, self.height)]
				else:
					result = [P(0, y) for y in range(self.height - 1, -1, -1)]
			if self.height == 1:
				assert (self.start.x == 0 or self.start.x == self.width - 1) and (self.end.x == 0 or self.end.x == self.width - 1)
				if self.start.x < self.end.x:
					result = [P(x, 0) for x in range(0, self.width)]
				else:
					result = [P(x, 0) for x in range(self.width - 1, -1, -1)]
		else:
			result = []
			if self.subboards[0].contains(self.subboards[0].rel2abs(self.start)):
				order = (0, 1)
			else:
				order = (1, 0)
			for i in order:
				relative_route = self.subboards[i].route()
				absolute_route = self.subboards[i].rel2abs(relative_route)
				result += absolute_route
		assert self.start == result[0] and self.end == result[len(result)-1]
		return result

"""
routes = []
for i in range(1000):
	random.seed(i)
	b = Board(7, 7, P(0, 0), P(6, 6))
	routes.append(b.route())
routes = sorted(routes, cmp = lambda a, b: int(hardness(a)*100000 - hardness(b)*100000))
for r in routes:
	print show_route(r).encode("utf8")
	print hardness(r)
"""


options, args = getopt.getopt(sys.argv[1:], '', ['width=', 'height=', 'text='])
if len(options) != 3:
	print "Usage: python wezykiem.py --width=8 --height=4 --text='To be or not to be - that is the question'"
	sys.exit(1)

for o, v in options:
	if o == '--width':
		width = int(v)
	if o == '--height':
		height = int(v)
	if o == '--text':
		text = v.decode('utf-8')

if len(text) < width * height:
	print "text is too short for this size"
	sys.exit(2)

start = P(0, 0)
corners = (P(width - 1, 0), P(0, height - 1), P(width - 1, height - 1))
end = random.choice([p for p in corners if (p.color() == start.color()) == ((width * height) % 2 == 1)])

board = Board(width, height, start, end)
route = board.route()
print show_route(route).encode("utf8")
print "\n"
print fill_route(route, text).encode("utf8")
